@@ -1,6 +1,6 @@
 /*
  * Pan - A Newsreader for X
- * Copyright (C) 1999  Pan Development Team (pan@superpimp.org)
+ * Copyright (C) 1999, 2000  Pan Development Team (pan@superpimp.org)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -23,12 +23,8 @@
 
 #include <gtk/gtkwidget.h>
 
-extern GtkWidget* article_toolbar_new (void);
+extern GtkWidget*       article_toolbar_new          (void);
 
-extern const gchar* article_toolbar_get_filter (void);
-
-/*extern void article_toolbar_set_sort_type (int sort_type, gboolean perform_sort);*/
-
-extern void article_toolbar_show_state_filter (gboolean);
+extern const gchar*     article_toolbar_get_filter   (void);
 
 #endif
