@@ -1,6 +1,6 @@
 /*
  * Pan - A Newsreader for X
- * Copyright (C) 1999  Pan Development Team (pan@superpimp.org)
+ * Copyright (C) 1999, 2000  Pan Development Team (pan@superpimp.org)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -29,16 +29,16 @@
 
 typedef struct
 {
-	const server_data *sdata;
-	const group_data *gdata;
-	article_data *adata;
-	GSList *list;
-	gboolean open;
-	gchar *filename;
-	StatusItem *item;
+	const Server  * server;
+	Group         * group;
+	GSList        * articles;
+	gboolean        open;
+	gchar         * path;
+	gchar         * filename;
+	StatusItem    * item;
 }
 decode_data;
 
-extern void decode_article (decode_data *);
+void decode_article   (const decode_data   * data);
 
 #endif /* __DECODE_H__ */
