@@ -1,6 +1,6 @@
 /*
  * Pan - A Newsreader for X
- * Copyright (C) 1999  Pan Development Team (pan@superpimp.org)
+ * Copyright (C) 1999, 2000  Pan Development Team (pan@superpimp.org)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,10 +21,8 @@
 #ifndef __LOG_H__
 #define __LOG_H__
 
+extern void log_add    (const char *);
+
 extern void log_add_va (const char *, ...);
-extern void log_add (const char *);
-extern void log_viewer_spawn (void);
-extern void log_init (void);
-extern void remove_crlf (gchar *msg);
 
 #endif /* __LOG_H__ */
