@@ -1,6 +1,6 @@
 /*
  * Pan - A Newsreader for X
- * Copyright (C) 1999  Pan Development Team (pan@superpimp.org)
+ * Copyright (C) 1999, 2000  Pan Development Team (pan@superpimp.org)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,27 +21,14 @@
 #ifndef __MESSAGE_SEND_H__
 #define __MESSAGE_SEND_H__
 
-#include <gtk/gtkwidget.h>
-#include "message.h"
-#include "sockets.h"
-#include "status-item.h"
+#include "server.h"
+#include "article.h"
+#include <glib.h>
 
-extern void send_cb (GtkWidget *widget, Message *msg);
+gint queue_article_for_posting (Server          * server,
+		                Article         * article,
+                                gboolean          check_article_first);
 
-enum MessageSendResult
-{
-	SMTP_FAILED	= (1<<0),
-	NNTP_FAILED	= (1<<1)
-};
-
-/**
- * @param StatusItem the item to which status/errors are emitted
- * @param sock the nntp socket to which nntp posts are sent
- * @param message the message to be posted/mailed
- * @return an or'ed list of MessageSendResult fields, or 0 on complete success
- *
- * FIXME: maybe it would be better to decouple email and nntp sends?  Ask Jason
- */
-extern gint message_send (StatusItem*, PanSocket *sock, Message *msg);
+void flush_sendlater_articles (void);
 
 #endif
