@@ -1,6 +1,6 @@
 /*
  * Pan - A Newsreader for X
- * Copyright (C) 1999  Pan Development Team (pan@superpimp.org)
+ * Copyright (C) 1999, 2000  Pan Development Team (pan@superpimp.org)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,6 +21,16 @@
 #ifndef __PRINT_H__
 #define __PRINT_H__
 
-void print_cb (GtkWidget *widget, gpointer data);
+#include <glib.h>
+#include <gtk/gtkwidget.h>
+
+void   print_cb                      (GtkWidget   * widget,
+                                      gpointer      data);
+
+#ifdef HAVE_GNOME_PRINT
+void   pan_gnome_print_cb            (void);
+
+void   pan_gnome_print_preview_cb    (void);
+#endif
 
 #endif
