@@ -1,6 +1,6 @@
 /*
  * Pan - A Newsreader for X
- * Copyright (C) 1999  Pan Development Team (pan@superpimp.org)
+ * Copyright (C) 1999, 2000  Pan Development Team (pan@superpimp.org)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -22,11 +22,19 @@
 #define __SAVE_H__
 
 #include <glib.h>
-
-extern void save_article (const gchar* message_id);
+#include "server.h"
+#include "group.h"
 
 extern void save_current_article (void);
 
-extern void save_current_article_cb (GtkWidget *widget, gpointer data);
+/**
+ * @param Group* the group to use when fetching the articles
+ *
+ * @param list a GSList* of GSList* of Article*.  Each sublist
+ *             constitutes a multipart message.  save_attachment_as
+ *             WILL FREE list, the sublist, and the Articles
+ *             when done.
+ */
+extern void save_attachment_as (Group*, GSList*);
 
 #endif /* __QUEUE_ITEM_BODY__H__ */
