@@ -1,6 +1,6 @@
 /*
  * Pan - A Newsreader for X
- * Copyright (C) 1999  Pan Development Team (pan@superpimp.org)
+ * Copyright (C) 1999, 2000  Pan Development Team (pan@superpimp.org)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -20,7 +20,6 @@
 
 #include <config.h>
 
-#include <ctype.h>
 #include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
@@ -31,14 +30,15 @@
 #include <netdb.h>
 
 #include <glib.h>
-#include <gtk/gtk.h>
 #include <libgnome/gnome-defs.h>
 #include <libgnome/gnome-i18n.h>
 #include <libgnome/gnome-config.h>
 
-#include "globals.h"
-#include "message.h"
-#include "message-window.h"
+#include <gmime/gmime-utils.h>
+
+#include "article.h"
+#include "gnksa.h"
+#include "prefs.h"
 #include "util.h"
 #include "smtp.h"
 #include "sockets.h"
@@ -59,7 +59,9 @@ extern const gchar* sockwrite_err_msg;
  * @return the received numeric value.
  */
 static int
-smtp_get_response (StatusItem *item, PanSocket *sock, int expected)
+smtp_get_response (StatusItem   * item,
+                   PanSocket    * sock,
+                   int            expected)
 {
 	const gchar *buf = NULL;
 	int retval = 0;
@@ -71,10 +73,10 @@ smtp_get_response (StatusItem *item, PanSocket *sock, int expected)
 
 	retval = atoi(buf);
 	if (expected>0 && retval!=expected)
-		status_item_emit_error (
+		status_item_emit_error_va (
 			item,
-			"Got unexpected response from mail server: "
-			"expected %d; got %s",
+			_("Got unexpected response from mail server: "
+			  "expected %d; got %s"),
 			expected,
 			buf);
 
@@ -91,11 +93,13 @@ smtp_get_response (StatusItem *item, PanSocket *sock, int expected)
  * @param string a string to write.
  * @return zero on success (no write errors), nonzero on failure.
  */
-static gint
-smtp_send_line (StatusItem *item, PanSocket *sock, const char* str)
+static int
+smtp_send_line (StatusItem   * item,
+                PanSocket    * sock,
+                const char   * str)
 {
 	int retval = 0;
-	if (Pan.mute)
+	if (pan_mute)
 	{
 		g_message ("(MUTE) to smtp: [%s]", str);
 		retval = 0;
@@ -120,8 +124,11 @@ smtp_send_line (StatusItem *item, PanSocket *sock, const char* str)
  * @param fmt a printf-style format string
  * @return zero on success (no write errors), nonzero on failure.
  */
-static gint
-smtp_send_line_va (StatusItem *item, PanSocket *sock, const char* fmt, ...)
+static int
+smtp_send_line_va (StatusItem  * item,
+                   PanSocket   * sock,
+                   const char  * fmt,
+                   ...)
 {
 	va_list ap;
 	int retval = 0;
@@ -153,107 +160,124 @@ smtp_send_line_va (StatusItem *item, PanSocket *sock, const char* fmt, ...)
  * @param socket the smtp socket where the message is mailed.
  * @param msg the message being sent.
  * @return zero on success, nonzero on failure
- *
- * FIXME: the tight coupling with the Message UI
- * FIXME: add wrap_long_lines after it's added to prefs
  */
 static int
-smtp_send_message (
-	StatusItem *item,
-	PanSocket *sock,
-	Message *msg,
-	const gchar* leader)
+smtp_send_article (StatusItem    * item,
+                   PanSocket     * sock,
+                   Article       * article,
+                   const gchar   * leader)
 {
-	gchar *from = NULL;
-	gchar *subject = NULL;
-	gchar *full_to = NULL;
-	gchar *reply_to = NULL;
-	gchar *body = NULL;
+	gchar * body = NULL;
+	gchar * from = NULL;
+	const gchar * reverse_path;
+	const gchar * reply_to;
+	const gchar * subject;
+	const gchar * mail_to;
+	const gchar * references;
 	gboolean okay = TRUE;
-	GSList *l = NULL;
 	gboolean maybe_okay = FALSE;
 
+	/* sanity checks */
 	g_return_val_if_fail (item!=NULL, -1);
 	g_return_val_if_fail (sock!=NULL, -1);
-	g_return_val_if_fail (msg!=NULL, -1);
-
-	if (!msg->rcpts) {
-		status_item_emit_error(item, _("No Recipients!"));
-		return -1;
-	}
+	g_return_val_if_fail (article!=NULL, -1);
 
 	status_item_emit_status (item, _("Preparing to Send Mail"));
 
 	/**
-	***  Generate headers
+	***  Generate/Locate headers
 	**/
 
-	subject = gtk_editable_get_chars (
-		GTK_EDITABLE (msg->window->subject), 0, -1);
-
-	body = gtk_editable_get_chars (
-		GTK_EDITABLE (msg->window->body), 0, -1);
-
-	from = gtk_editable_get_chars (
-		GTK_EDITABLE (msg->window->from), 0, -1);
-	if (!from)
-		from = gnome_config_get_string ("/Pan/User/Email");
-	if (!from) {
-		status_item_emit_error(item, _("No Sender Specified!"));
+	/* who is this mail to? */
+	mail_to = article_get_header (article, PAN_MAIL_TO);
+	pan_warn_if_fail (is_nonempty_string (mail_to));
+	if (!is_nonempty_string(mail_to)) {
+		status_item_emit_error (item, _("No Recipients!"));
 		okay = FALSE;
 	}
 
-	l = msg->rcpts;
-	while (l) {
-		if (full_to)
-			full_to = g_strconcat (full_to, (gchar*)l->data, NULL);
-		else
-			full_to = g_strconcat ((gchar*)l->data, NULL);
-		if (l->next)
-			full_to = g_strconcat (full_to, ", ", NULL);
-		l = l->next;
+	/* who is this mail from?  (part 1) */
+	reverse_path = NULL;
+	if (okay) {
+		reverse_path = article_get_header (article, PAN_REVERSE_PATH);
+		if (!is_nonempty_string(reverse_path)) {
+			g_message (_("Can't parse the From: line; hope for the best."));
+			reverse_path = article->author_addr;
+		}
+		if (!is_nonempty_string(reverse_path)) {
+			pan_warn_if_reached ();
+			okay = FALSE;
+		}
 	}
 
-	/* Build "Reply-To: " contents */
-        if (GTK_IS_ENTRY(msg->window->reply_to))
-		reply_to = gtk_editable_get_chars (GTK_EDITABLE (msg->window->reply_to), 0, -1);
+	/* who is this mail from?  (part 2) */
+	from = NULL;
+	if (okay) {
+		gchar * author = article_get_author_str (article);
+		if (is_nonempty_string (author))
+			from = g_strdup (author);
+		if (from == NULL)
+			from = gnome_config_get_string ("/Pan/User/Email");
+		if (from == NULL) {
+			status_item_emit_error(item, _("No Sender Specified!"));
+			okay = FALSE;
+		}
+		g_free (author);
+	}
 
 	/**
 	*** start sending a letter
 	**/
-
 	if (okay)
 		status_item_emit_status (item, _("Sending Mail"));
 
 	if (okay) {
-		const char *fmt = "MAIL FROM: <%s>\r\n";
-		okay = !smtp_send_line_va (item, sock, fmt, from);
+		const char *fmt = "MAIL FROM:<%s>\r\n";
+		okay = !smtp_send_line_va (item, sock, fmt, reverse_path);
 	}
 	if (okay)
-		okay = Pan.mute || smtp_get_response(item, sock, 250)==250;
+		okay = pan_mute || smtp_get_response(item, sock, 250)==250;
 
 	/**
 	*** tell the server who we're sending the letter to
 	**/
 
 	maybe_okay = FALSE;
-	for (l=msg->rcpts; okay && l!=NULL; l=l->next)
+	if (okay)
 	{
-		if (okay)
+		guint i;
+		GPtrArray* to_array = gnksa_split_addresses (mail_to);
+		for (i=0; i!=to_array->len; ++i)
 		{
-			const char *fmt = "RCPT TO: <%s>\r\n";
-			okay = !smtp_send_line_va (item, sock, fmt, l->data);
-		}
-		if (okay)
-		{
-			const int retval = Pan.mute ? 250 : smtp_get_response(item, sock, -1);
+			if (okay)
+			{
+				const gchar * fmt = "RCPT TO: <%s>\r\n";
+				const char * name = (const char*) g_ptr_array_index (to_array, i);
+				gchar * address;
+				gchar * realname;
 
-			/* >=1 known recepient */
-			maybe_okay = retval==250;
+				gnksa_do_check_from (name, &address, &realname);
+				okay = !smtp_send_line_va (item, sock, fmt, address);
 
-			/* keep going even if one recipient is unknown */
-			okay = retval==250 || retval==550;
+				g_free (address);
+				g_free (realname);
+			}
+			if (okay)
+			{
+				const int retval = pan_mute
+					? 250
+					: smtp_get_response(item, sock, -1);
+
+				/* >=1 known recepient */
+				maybe_okay = retval==250;
+
+				/* keep going even if one recipient is unknown */
+				okay = retval==250 || retval==550;
+			}
 		}
+
+		pan_g_ptr_array_foreach (to_array, (GFunc)g_free, NULL);
+		g_ptr_array_free (to_array,  TRUE);
 	}
 
 	/* don't proceed unless we have at least one known recipient */
@@ -271,47 +295,78 @@ smtp_send_message (
 	if (okay)
 		okay = !smtp_send_line (item, sock, "DATA\r\n");
 	if (okay)
-		okay = Pan.mute || smtp_get_response (item, sock, 354)==354;
+		okay = pan_mute || smtp_get_response (item, sock, 354)==354;
 
 	/**
 	*** send the message
 	**/
+        
+	references = article_get_header (article, HEADER_REFERENCES);
 
-	if (okay)
-		okay = !smtp_send_line_va (item, sock, "X-Mailer: Pan %s\n", VERSION);
-	if (okay)
-		okay = !smtp_send_line_va (item, sock, "From: %s\n", from);
-	if (okay)
-		okay = !smtp_send_line_va (item, sock, "To: %s\n", full_to);
-	if (okay)
-		okay = !smtp_send_line_va (item, sock, "Subject: %s\n", subject);
-	if (okay && reply_to)
-		okay = !smtp_send_line_va (item, sock, "Reply-To: %s\n", reply_to);
+	reply_to = article_get_header (article, HEADER_REPLY_TO);
+
+	subject = article->subject;
+
+	body = g_strstrip (article_get_body (article));
+
+
+	if (okay) {
+		gchar * tmp = g_strdup ("User-Agent: Pan/" VERSION " (Unix)");
+		replace_gstr (&tmp, g_mime_utils_8bit_header_encode (tmp));
+		okay = !smtp_send_line_va (item, sock, "%s\r\n", tmp);
+		g_free (tmp);
+	}
+	if (okay) {
+		gchar * tmp = g_mime_utils_8bit_header_encode (from);
+		okay = !smtp_send_line_va (item, sock, "From: %s\r\n", tmp);
+		g_free (tmp);
+	}
+	if (okay) {
+		gchar * tmp = g_mime_utils_8bit_header_encode (mail_to);
+		okay = !smtp_send_line_va (item, sock, "To: %s\r\n", tmp);
+		g_free (tmp);
+	}
+	if (okay) {
+		gchar * tmp = g_mime_utils_8bit_header_encode (subject);
+		okay = !smtp_send_line_va (item, sock, "Subject: %s\r\n", tmp);
+		g_free (tmp);
+	}
+	if (okay && references!=NULL) {
+		gchar * tmp = g_mime_utils_8bit_header_encode (references);
+		okay = !smtp_send_line_va (item, sock, "References: %s\r\n", tmp);
+		g_free (tmp);
+	}
+	if (okay && reply_to) {
+		gchar * tmp = g_mime_utils_8bit_header_encode (reply_to);
+		okay = !smtp_send_line_va (item, sock, "Reply-To: %s\r\n", tmp);
+		g_free (tmp);
+	}
 	if (okay)
 		okay = !smtp_send_line (item, sock, "\r\n" );
 	if (okay && leader && *leader)
-		okay = !smtp_send_line_va (item, sock, "%s\n\n", leader);
+		okay = !smtp_send_line_va (item, sock, "%s\r\n\r\n", leader);
 	if (okay) {
 		int i;
-		gchar **sd = g_strsplit (body, "\n", -1);
+		gchar ** sd;
+		GString * tmp = g_string_sized_new (120);
+		replace_gstr (&body, pan_substitute(body,"\r\n","\n"));
+	       	sd = g_strsplit (body, "\n", -1);
 		for (i=0; sd[i]!=NULL; ++i) {
-	 		if (okay) okay = !smtp_send_line (item, sock, sd[i] );
-			if (okay) okay = !smtp_send_line (item, sock, "\n" );
+			g_string_sprintf (tmp, "%s\r\n", sd[i]);
+	 		if (okay) okay = !smtp_send_line (item, sock, tmp->str);
 		}
 		g_strfreev (sd);
+		g_string_free (tmp, TRUE);
 	}
 	if (okay)
 		okay = !smtp_send_line (item, sock, ".\r\n");
 	if (okay)
-		okay = Pan.mute || smtp_get_response (item, sock, 250)==250;
+		okay = pan_mute || smtp_get_response (item, sock, 250)==250;
 	if (okay)
 		status_item_emit_status (item, _("Mail Sent!"));
 	
 	/* clean up */
 	g_free (from);
-	g_free (full_to);
-	g_free (reply_to);
-	g_free (subject);
 	g_free (body);
 
 	return okay ? 0 : -1;
@@ -320,7 +375,8 @@ smtp_send_message (
 
 /* begin a SMTP session */
 static int
-smtp_handshake (StatusItem *item, PanSocket *sock)
+smtp_handshake (StatusItem   * item,
+                PanSocket    * sock)
 {
 	gchar my_name[256+1];
 	struct hostent *hp;
@@ -333,7 +389,7 @@ smtp_handshake (StatusItem *item, PanSocket *sock)
 
 	if (smtp_send_line_va (item, sock, "HELO %s\r\n", my_name))
 		return -1;
-	if (!Pan.mute && smtp_get_response (item, sock, 250) != 250)
+	if (!pan_mute && smtp_get_response (item, sock, 250) != 250)
 		return -1;
 
 	return 0;
@@ -342,15 +398,14 @@ smtp_handshake (StatusItem *item, PanSocket *sock)
 
 /* end a SMTP session */
 static void
-smtp_disconnect (StatusItem *item, PanSocket *sock)
+smtp_disconnect (StatusItem   * item,
+                 PanSocket    * sock)
 {
 	if (sock != NULL)
 	{
-		//smtp_send_line (item, sock, ".\r\n");
-
 		smtp_send_line (item, sock, "QUIT\r\n");
 
-		if (!Pan.mute)
+		if (!pan_mute)
 			smtp_get_response (item, sock, 221);
 	}
 }
@@ -360,37 +415,31 @@ static PanSocket*
 smtp_get_connection (StatusItem *item)
 {
 	PanSocket *sock = NULL;
-	gchar *mailhost = NULL;
-	gint mailport = -1;
 
 	/* get smtp address/port */
-	mailhost = gnome_config_get_string ("/Pan/Mail/smtp_address=localhost");
-	mailport = gnome_config_get_int ("/Pan/Mail/smtp_port=25");
 	status_item_emit_status (item, _("Connecting to mail server"));
 
 	/* get the socket... */
-	sock = pan_socket_new (mailhost, mailport);
+	sock = pan_socket_new (mail_server_address, mail_server_port);
+	pan_object_ref (PAN_OBJECT(sock));
 	pan_object_sink (PAN_OBJECT(sock));
-	if (sock->sockfd == 0) {
-		status_item_emit_error (item,
-			_("Unable to open connection to mail server ``%s''"),
-			mailhost);
-		g_free (mailhost);
+	if (sock->sockfd < 0) {
+		status_item_emit_error_va (item,
+			_("Unable to open connection to mail server \"%s\""),
+			mail_server_address);
 		pan_object_unref (PAN_OBJECT(sock));
 		sock = NULL;
-		return NULL;
 	}
-	g_free (mailhost);
 
 	/* get the "ready" message from the socket... */
-	if (smtp_get_response (item, sock, 220) != 220) {
-		status_item_emit_error (item, _("Mail Server Not Ready"));
+	if (sock!=NULL && smtp_get_response (item, sock, 220) != 220) {
+		status_item_emit_error_va (item, _("Mail Server Not Ready"));
 		pan_object_unref (PAN_OBJECT(sock));
 		sock = NULL;
 	}
 
 	/* handshake */
-	if (smtp_handshake (item, sock)) {
+	if (sock!=NULL && smtp_handshake (item, sock)) {
 		pan_object_unref (PAN_OBJECT(sock));
 		sock = NULL;
 	}
@@ -404,15 +453,17 @@ smtp_get_connection (StatusItem *item)
 ****
 ***/
 
-gint
-smtp_send (StatusItem *item, Message *msg, const gchar* leader)
+int
+smtp_send (StatusItem      * item,
+           Article         * article,
+           const gchar     * leader)
 {
-	PanSocket *sock = NULL;
+	PanSocket * sock = NULL;
 	int retval;
 
 	/* sanity checks */
 	g_return_val_if_fail (item!=NULL, -1);
-	g_return_val_if_fail (msg!=NULL, -1);
+	g_return_val_if_fail (article!=NULL, -1);
 
 	/* connect to the smtp server */
 	sock = smtp_get_connection (item);
@@ -420,7 +471,7 @@ smtp_send (StatusItem *item, Message *msg, const gchar* leader)
 		return -1;
 
 	/* send the message */
-	retval = smtp_send_message (item, sock, msg, leader);
+	retval = smtp_send_article (item, sock, article, leader);
 
 	/* say goodbye and disconnect */
 	smtp_disconnect (item, sock);
