@@ -1,6 +1,6 @@
 /*
  * Pan - A Newsreader for X
- * Copyright (C) 1999  Pan Development Team (pan@superpimp.org)
+ * Copyright (C) 1999, 2000  Pan Development Team (pan@superpimp.org)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,16 +21,18 @@
 #ifndef __SMTP_H__
 #define __SMTP_H__
 
-#include "message.h"
+#include "article.h"
 #include "status-item.h"
 
 /**
  * Opens a new smtp socket and sends the specified message out.
  *
- * @param status the StatusItem through which status and error messages will be emitted.
+ * @param status the StatusItem through which status and error messages go.
  * @param message the message to be sent
  * @param leader an optional line to prepend the email body.
  */
-extern gint smtp_send (StatusItem *status, Message *msg, const gchar* leader);
+int smtp_send (StatusItem   * status,
+               Article      * article,
+               const char   * leader);
 
 #endif
